# -*- coding: utf-8 -*-
"""
    pip_services3_mongodb.build.__init__
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

__all__ = ['DefaultMongoDbFactory']

from .DefaultMongoDbFactory import DefaultMongoDbFactory
