# -*- coding: utf-8 -*-
"""
    pip_services3_mongodb.connect.__init__
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    MongoDbConnectionResolver module initialization.
    
    :copyright: Conceptual Vision Consulting LLC 2018-2019, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

__all__ = ['MongoDbConnectionResolver', 'MongoDbConnection']

from .MongoDbConnection import MongoDbConnection
from .MongoDbConnectionResolver import MongoDbConnectionResolver
