# -*- coding: utf-8 -*-
"""
    pip_services3_mongodb.__init__
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Mongodb module initialization
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

__all__ = [ ]


